/* File: BURNMODL.CPP */

/* The tridiagonal solver and associated .h files were written by:
     Numerical Recipies in C; The Art of Scientific Computing; Second Edition; William H. Press et al.; Cambridge University Press 
		Copyright (C) 1986 - 1995 */
		
#include "burn.h"
#include "strings.h"

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  Main
 *	
 *	Created:	4/1/98	SJR
 *	Modified:	7/22/98	Modification:	added call to JetConvection function
 *	Modified:	9/4/98	Modification:	added memory allocation code for Temperature matrix
 *	Modified:	9/14/98	Modification:	changed Omega calculation completely
 *	Modified:	9/17/98 Modification:	changed code for T boundary conditions to change with time steps
 *
 *	Local Variables:	int i, j, d:									looping and place marking
 *						int fnode:										flag for printing Omega at requested depth
 *						double depth:									used for interpolation of depth where Omega = 1
 *						long kpm, nrows, ncols, row, col:				used for memory allocation of Temperature matrix
 *						char bc[10] = "LR":								used for switch on boundary conditions
 *						double *ppdiag,*ppsuperd,*ppsubd,*pprhs,*ppu:	pointers for tridag(); diag, upper, lower, rhs, T matrices 
 *						unsigned long c, k, n:							necessary for tirdag() 
 *						char junk[85]:									used for reading string and temporary string assignment
 *						char directory[20]:								used to re-assign the main() input argument
 *						struct datapair omegaray[MAXARRAY]:				used to array all values of Omega and their depths
 *						FILE *fpout, *fpomega:							file pointers
 *	
 *	Global Variables modified: double upper[], diag[], lower[], rhs[] 
 *
 * ------------------------------------------------------------------------- */
main (int argc, char *argv[])
{

	int i, j, d, fnode;
	double depth;
	long kpm, nrows, ncols, row, col;
	char bc[10] = "LR";								
	double *ppdiag,*ppsuperd,*ppsubd,*pprhs,*ppu;	
	unsigned long c, k, n;							
	char junk[85], directory[20];					
	struct datapair omegaray[MAXARRAY];
	FILE *fpout, *fpomega;

/* copy the input argument into a temporary variable and then to datadir */
	strcpy (directory, argv[1]); 
	strcpy (datadir, directory);

/* output the title block to the screen */
	Title (); 

/* call GetData() */
	if (!GetData ()) 
	{
		printf("GetData() Error: %s", GETDATAERR); printf("%s", BLANKLINE); return NULL; 
	}  

	if (BATCH == 0)		/* if true: user has to step through program with Enter key */
	{
		printf("%s", ENDOFSEG);		/* notify: end of title, start of solver */
		(void) getchar();		
		for (i=1;i<=30;i++) printf("%s", BLANKLINE);
	}

	if (lbc_type[0] == 'C' && jet == 1)		/* if true: calculate the initial HBAR */
	{
		jetdur[1].y = JetConvection(0, T[1].y, lbc2[1].y); 
	}

/* allocate the memory for the array */
	ncols = nodes + 1;
	nrows = time_steps + 1;

	aptr = (double *)malloc((size_t)(((nrows+1) * (ncols+1) )*sizeof(double)));
	if(aptr == NULL)
	{
		printf("Memory Error: %s", MEMERROR1);
		printf("Memory Error: %s", MEMCHECK1);
		exit(0);
	}
/* allocate the memory for the pointers to the rows */
	rptr = (double **)malloc((size_t)((nrows+1) * sizeof(double*)));
	if(rptr == NULL)
	{
		printf("Memory Error: %s", MEMERROR2);
		printf("Memory Error: %s", MEMCHECK2);
		exit(0);
	}
/* 'point' the pointers to the data*/
	
	for(kpm = 0; kpm < nrows+1; kpm++)
	{
		rptr[kpm] = aptr + (kpm * ncols);
	}

/* call Fourier() */
	if (!Fourier ())				
	{
		printf("Fourier() Error: %s", FOURIERROR); printf("%s", BLANKLINE); return NULL; 
	} 

/* call InterNodes() */
	if (!InterNodes ())					
	{
		printf("InterNodes() Error: %s", INTNODERROR); printf("%s", BLANKLINE); return NULL; 
	} 

/* output A-Matrix and Initial Node Temperature Matrix */
	if (VERBOSE == 1)		/* if true: print the following to the output file */ 								
	{
		printf("%s", STARTPROG);	 
		for (i=1;i<=20;i++) printf("%s", BLANKLINE);
		sprintf (junk, "%sOUTPUT.TXT", datadir);
	/* open and append datafile */
		if ((fpout=fopen(junk, "a")) == NULL) { printf("Main() Error: %s %s", MAINERROR1, MAINCHECK1); return (0); }
		fprintf(fpout, "\r\n\r\nTRIAL:%5d", trial);		
		fprintf(fpout, "    Execution Date: %8s", date);
		fprintf(fpout, "    Executed By: %6s\r\n", inits);
		if (jet) fprintf(fpout, "\r\nHbar duration = %lf\r\n", dur);
		if (printA)		/* if true:  output the A-matrix */
		{
			fprintf(fpout, "A-Matrix Diagonals\r\n");	
			fprintf(fpout, " Lower      Middle     Upper  \r\n");  
			fprintf(fpout, "           %20.12f, %20.12f\r\n", diag[1], upper[1]);	
			for (j= 2;j<nodes;j++) fprintf(fpout, "%20.12f, %20.12f, %20.12f\r\n", lower[j], diag[j], upper[j]);
			fprintf(fpout, "%20.12f, %20.12f\r\n", lower[nodes], diag[nodes]);
		}
		if (printI)		/* if true:  output the initial temperature distribution */								
		{
			fprintf(fpout, "\n   Node   Depth               T(K):  Initial Temperature Distribution\n");	
			for (i=1;i<=nodes;i++) fprintf(fpout, "%7d     %9.7f   %10.1f\r\n",i, (i*dx - dx), T[i].y);	
		}
		fclose(fpout);		/* close the file pointer fpout */
	}
	else if (VERBOSE ==	0) {		/* if true: print the following to the screen */
		d = 1;			
		if (printA)		/* if true:  output the A-matrix */
		{
			printf("%s", AMATRIX1); printf("%s", BLANKLINE);
			printf("%s", AMATRIX2); printf("%s", BLANKLINE); 
			printf("           %20.12f, %20.12f\r\n", diag[1], upper[1]);
			for (j= 2;j<nodes;j++) 
			{
				printf("%20.12f, %20.12f, %20.12f\r\n", lower[j], diag[j], upper[j]);
				if (j == d*20) 
				{
					(void) getchar();  d += 1;
				}
			}
			printf("%20.12f, %20.12f\r\n", lower[nodes], diag[nodes]);
			printf("%s", AMATRIX3);  (void) getchar();
		}
		if (printI)		/* if true:  output the initial temperature distribution */
		{
			d = 1;
			printf("%s", BLANKLINE); printf("%s", ITEMP1); printf("%s", BLANKLINE);
			for (i=1;i<=nodes;i++) 
			{ 
				printf("%3d     %9.7f  %10.1f\r\n",i, (i*dx - dx), T[i].y);
				if (i == d*20) 
				{
					(void) getchar();  d += 1;
				}
			}
			printf("%s", ITEMP2);  (void) getchar();
		}
		for (i=1;i<33;i++) printf("%s", BLANKLINE);
		printf("%s", ITEMP3); 
		for (i=1;i<13;i++) printf("%s", BLANKLINE);
	
	}
	else		/* output error if VERBOSE is not set to 1 or 0  */
	{ 
		printf("Main() Error: %s %s", MAINERROR2, MAINCHECK2 );  
		printf("%s%s", BLANKLINE, BLANKLINE); (void) getchar();
		return(0);   
		
	}
/* loop to calculate T matrix solutions based on the L or R BC type and time_step */
	for (i=1;i<=time_steps;i++)
	{

		switch(lbc_type[0])		/* switch: based on L side heat transfer type */
		{
			case 'C': { Convection(bc[0], i); }		/* call Convection() */
				break;
			case 'F': { Flux(bc[0], i);  }			/* call Flux() */
				break;
			case 'T': { diag[1] = 1.000;  upper[1] = 0.000;	 rhs[1] = lbc1[i+1].y; } /* set coefficients for LBC Constant Temperature */
				break;
			default: { printf("Main() Error: %s %s", MAINERROR3, MAINCHECK3); printf("%s", BLANKLINE);  
					   (void) getchar(); return (0);}		/* output error if lbc_type is not set to C, F, or T */
				break;
		}
		
		switch(rbc_type[0]) 
		{
			case 'C': { Convection(bc[1], i); }		/* call Convection() */
				break;
			case 'F': { Flux(bc[1], i); }			/* call Flux() */
				break;
			case 'T': { diag[nodes] = 1.000;  lower[nodes] = 0.000;	 rhs[nodes] = rbc1[i+1].y; } /* set coefficients for RBC Const. Temp.*/
				break;
			default: { printf("Main() Error: %s %s", MAINERROR4, MAINCHECK4); printf("%s", BLANKLINE);
					   (void) getchar(); return (0);} 
				break;
		}

/* call InterRHS() */
		InterRHS();

/* call vector() */
		ppdiag=vector(1,NP);
		ppsuperd=vector(1,NP);
		ppsubd=vector(1,NP);
		pprhs=vector(1,NP);
		ppu=vector(1,NP);

		n = nodes;		/* set the proper data type for tridiagonal solver */

/* set tridiagonal pointers equal to calculated global variables */
		for (k=1;k<=n;k++) ppdiag[k] = diag[k];		
		for (k=1;k<n;k++) ppsuperd[k] = upper[k];	
		for (k=2;k<=n;k++) ppsubd[k] = lower[k];	
		for (k=1;k<=n;k++) pprhs[k] = rhs[k];		

/* call tridag() */
		tridag(ppsubd,ppdiag,ppsuperd,pprhs,ppu,n);

/* fill the temperature matrix row with results of tridiagonal solver */
		row = i;		/* set the row to be filled equal to the current time step */
		
		for(col = 1; col <= ncols-1; col++)		/* loop through columns of matrix */
		{
			rptr[row][col] = ppu[col];
		}
		
/* output the temperature matrix */

		if (VERBOSE == 1)		/* if true:  print the following to the output file */
		{
			sprintf (junk, "%sOUTPUT.TXT", datadir);
			if ((fpout=fopen(junk, "a")) == NULL) {printf("Main() Error: %s", MAINERROR1); return (0); }
			if (printtime)		/* if true: output Temp vs. Depth */
			{
				if (i*dt == printtime)		/* if true: output the temp vs. depth curve at this time */
				{
					fprintf(fpout, "\nNode    Depth           T(K) @ time %10.5f sec\n", (i*dt));
					for (k=1;k<=n;k++) fprintf(fpout, "%3d     %9.7f  %10.1f\r\n",k, (k*dx - dx), ppu[k]);
					if (jet) jetdur[i+1].y = JetConvection(i, ppu[1], lbc2[i+1].y);				/* if true: calc. convection coefficient */
				}
			}
			else if (printnode)		/* if true: output Temp vs. time */
			{
				k = printnode;		/* set k = to desired node to reduce code length */
				if (i < 2) fprintf(fpout, "\nTime Step  Depth(m)     time(sec)       T(K) @ Node %3d\n", k);
				fprintf(fpout, "%7d      %9.7f %10.5f   %10.1f\r\n", i, (k*dx-dx),i*dt,ppu[k]);
				Tcritdepth[i].x = i*dt;		Tcritdepth[i].y = ppu[k];    /* set critical depth values; used only when multio is set to 0 */
				if (jet) jetdur[i+1].y = JetConvection(i, ppu[1], lbc2[i+1].y);			/* if true: calc convection coefficient using surface temp */
			}
			else	/* if printtime and printnode are both 0 print all output to file */
			{
				fprintf(fpout, "\nNode    Depth           T(K) @ time %10.5f sec\n", (i*dt));
				for (k=1;k<=n;k++) 
				{
					fprintf(fpout, "%3d     %9.7f  %10.1f\r\n",k, (k*dx - dx), ppu[k]);	/* print all nodes at all time steps into file */
					if (jet) jetdur[i+1].y = JetConvection(i, ppu[1], lbc2[i+1].y);			
				}
			}

		fclose(fpout);		/* close file pointer fpout */

		if (i == time_steps && BATCH == 0) { printf("%s", ENDOUTPUT);		/* output message to inform user that file has been created */
		(void) getchar(); }

		}
		else if (VERBOSE == 0) {		/* if true: print the following to the screen */
			if (printtime) {		
				if (i*dt == printtime) {
				printf("\nNode    Depth           T(K) @ time %10.5f sec\n", (i*dt));
					for (k=1;k<=n;k++) {
						printf("%3d     %9.7f  %10.1f\r\n",k, (k*dx - dx), ppu[k]);
					}
				(void) getchar(); 
				} 
			}
			else if (printnode) {
				k = printnode;
				if (i < 2) {printf("\nDepth(m)     time(sec)       T(K) @ Node %3d\n", k);  d = 1;}
				printf("%9.7f     %10.5f   %10.1f\r\n", (k*dx-dx),i*dt,ppu[k]); 
				Tcritdepth[i].x = i*dt;	  	Tcritdepth[i].y = ppu[k];  
				if (i == d*20) {(void) getchar();   d += 1; } 
			}
			else { 
				printf("\nNode    Depth           T(K) @ time %10.5f sec\n", (i*dt));
				for (k=1;k<=n;k++) { 
					printf("%3d     %9.7f  %10.1f\r\n",k, (k*dx - dx), ppu[k]);
					if (k == c*20) {(void) getchar();   c += 1; }
				}
				(void) getchar();
			}
		}
		else {		/* output error if VERBOSE is not set to 1 or 0 */
			printf("Main() Error: %s", VERBOSERROR);  
			printf("%s%s", BLANKLINE, BLANKLINE);  return(0); }

/* call free_vector() */
		free_vector(ppu,1,NP);
		free_vector(pprhs,1,NP);
		free_vector(ppsubd,1,NP);
		free_vector(ppsuperd,1,NP);
		free_vector(ppdiag,1,NP);	


	}	/* end of loop based on time_steps */


/* start of the burn integral computation */ 

fnode = printnode;		/* set fnode to hold original value of printnode */

	if (BATCH == 0)			 /* if true: notify user that the Burn Integral will now run */
	{
		for (i=1;i<=22;i++) printf("%s", BLANKLINE);
		printf("%s", STARTOMEGA);
		for (i=1;i<=12;i++) printf("%s", BLANKLINE);
		printf("%s", ENDOFSEG);		 
		(void) getchar();	
	}

	if (printnode && multio == 0)		/* if true:  find only the one value of Omega at printnode */
	{	
		omegaray[1].y = BurnInjury();	/* set value equal to return value of BurnInjury() */
		if (!omegaray[1].y)		/* if true:  output error that Omega wasn't properly calculated */
		{ 
			printf("Main() Error: %s", MAINERROR5);  printf("%s%s", BLANKLINE, BLANKLINE);
			(void) getchar(); 
			return NULL;
		} 
		printf("Omega at node %d = %20.12lf\r\n", printnode, omegaray[1].y);		/* output Omega at printnode to screen */
		if (VERBOSE == 1)		/* if true: print the following to the output file */
			{
				sprintf (junk, "%sOUTPUT.TXT", datadir);
				if ((fpomega=fopen(junk, "a")) == NULL) return (0); 
				printf("Omega at node %d = %20.12lf\r\n", printnode, omegaray[1].y);		/* output Omega to file */
				fclose(fpomega);		/* close file pointer fpomega */
			}
		
	}
	else		/* use the following code to find Omega for all other input cases */
	{
		for (col = 1; col < ncols; col++)		/* loop based on the column in the matrix */
		{
			for (row=1;row<=time_steps;row++)		/* loop based on the row in the matrix */
			{
				Tcritdepth[row].x = row * dt;		/* set critical depth values */
				Tcritdepth[row].y = rptr[row][col];
			}
			printnode = col;		/* update printnode to the current column */
			omegaray[col].y = BurnInjury();		/* set array value to the returned value of BurnInjury() */
		}
		
		if (omegaray[1].y > one && omegaray[nodes].y > one)		/* if true: all values of Omega are greater than one */
		{
			for (j=1;j<=35;j++) printf("%s", BLANKLINE);
			printf("Omega Output: %s", NOMEGA1);
			(void) getchar();
			exit(0);
		}
		else if (omegaray[1].y < one && omegaray[nodes].y < one)		/* if true: all values of Omega are less than one */
		{
			for (j=1;j<=35;j++) printf("%s", BLANKLINE);
			printf("Omega Output: %s", NOMEGA2);
			(void) getchar();
			exit(0);
		}
		else		/* all other cases use the following code */
		{
			for (i = 1; i<=nodes; i++)		/* loop through all values of Omega  */
			{
				if (omegaray[i].y > 1.00 && omegaray[i+1].y < 1.00)		/* if true: Omega has been found to be between these nodes */
				{
					depth = GenInt(omegaray[i+1].y, one, omegaray[i].y, ((i+1)*dx), (i*dx));		/* set depth equal to return value of GenInt() */
					for (j=1;j<=15;j++) printf("%s", BLANKLINE);		/* output Omega results to screen */
					printf("Omega at depth %lf (m) = %20.12lf\r\n", (i*dx), omegaray[i].y);
					printf("Omega at depth %lf (m) = %20.12lf\r\n", ((i+1)*dx), omegaray[i+1].y);
					printf("By interpolation, Omega is equal to 1 at depth %lf (m).\r\n", depth);
					for (j=1;j<=15;j++) printf("%s", BLANKLINE);
					if (VERBOSE == 1)		/* if true: output the following to output file */
					{
						sprintf (junk, "%sOUTPUT.TXT", datadir);
						if ((fpomega=fopen(junk, "a")) == NULL) return (0); 
						fprintf(fpomega, "Omega at node %d = %20.12lf\r\n", fnode, omegaray[fnode].y);
						fprintf(fpomega, "Omega = 1 at depth %10.6lf (m)\r\n", depth);
						fclose(fpomega);
					}
				}
				else if (omegaray[i].y < 1.00 && omegaray[i+1].y > 1.00)		/* if true:  Omega has been found to be between these nodes */
				{
					depth = GenInt(omegaray[i].y, one, omegaray[i+1].y, (i*dx), ((i+1)*dx));
					for (j=1;j<=15;j++) printf("%s", BLANKLINE);
					printf("Omega at depth %lf (m) = %20.12lf\r\n", (i*dx), omegaray[i].y);
					printf("Omega at depth %lf (m) = %20.12lf\r\n", ((i+1)*dx), omegaray[i+1].y);
					printf("Therefore, by interpolation, Omega is equal to 1 at depth %lf (m).\r\n", depth);
					for (j=1;j<=15;j++) printf("%s", BLANKLINE);
					if (VERBOSE == 1) 						
					{
						sprintf (junk, "%sOUTPUT.TXT", datadir);
						if ((fpomega=fopen(junk, "a")) == NULL) return (0); 
						fprintf(fpomega, "Omega at node %d = %20.12lf\r\n", fnode, omegaray[fnode].y);
						fprintf(fpomega, "Omega = 1 at depth %lf (m)\r\n", depth);
						fclose(fpomega);
					}
				}
				else continue;		/* if omega is not between the two current nodes increment the nodes */
			}
		}
		
	}
	
/* release the memory for the temperature matrix */
	free(rptr);
	free(aptr);
	
	if (BATCH == 0)		/*  if true:  signal end of program */
	{
		printf("%s", ENDPROG); 
		(void) getchar();
	}
	
return NULL;

}	/* End of Main() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void title(void)
 *
 *	Purpose:	print the title block to the screen
 *
 *	Arguments:	void
 *	Return:		void
 *	
 *	Created:	7/28/98	SJR
 *
 *	Local Variables:	int i - used in 'for' loops
 *	
 * ------------------------------------------------------------------------- */

void Title(void) 
{
	int i;

	for (i=1;i<6;i++) printf("%s", BLANKLINE);

	printf("\t\t\t  UMTRI Airbag Skin Burn Model\r\n");
	printf("\t\t\t\t  Release 9809\r\n");

	for (i=1;i<4;i++) printf("%s", BLANKLINE);

	printf("\t\t\t\t Programmed by:\r\n\r\n");
	printf("\t\t\t     University of Michigan\n");
	printf("\t\t\tTransportation Research Institute\n");
	printf("\t\t\t     Biosciences Division\n");
	printf("\t\t\t\t Steven J. Reed\n");
	printf("\t\t\t     email: sjreed@umich.edu\n");

	for (i=1;i<10;i++) printf("%s", BLANKLINE);
   
}	/* End of Title() */


/* ----------------------------------------------------------------------------
 *
 *	Function Name:  int GetData(void)
 *
 *	Purpose:		Collect data from DATA.DAT, call for interpolation of variables,
 *					set program controls based on inputs
 *
 *	Arguments:	None
 *
 *	Return:  1 if successful, 0 if DATA.DAT file doesn't open
 *	
 *	Created:	4/1/98	SJR
 *	Modified:			Modification: add sort function call to sort datapairs
 *	Modified:			Modification: removed t acquisition from data file	
 *  Modified:	5/25/98 Mod:  changed header in DATA.DAT to have no limit to number of lines; as long as there's an *
 *				7/7/98	Mod:  changed * to # in input DATA.DAT for header junk variable 
 *				7/7/98	Mod:  removed loop that verified TIME_STEPS > NODES:  proved to be unnecessary
 *				7/7/98	Mod:  added functionality to call another datafile for input data sets
 *				9/2/98  Mod:  Added additional error checking for data file read
 *				9/17/98	Mod:  Additional error checking, added h variable
 *
 *	Local Variables:	int g, h, i, j, k, m, err:			- used for loops
 *						char property[20]	- used in switches
 *						char junk[85]		- temporary string storage
 *						char *pprop			- =property[0]; pass to Interp
 *						double tardD:		- target diameter, used for impinging jet only
 *						FILE *fpdata;		- used to open DATA.DAT
 *						FILE *fpinter;		- used to open any external data file where j is equal to 0 in DATA.DAT
 *						FILE *fpimpjet;		- used to open LEFTIMP.DAT if j is -1 in DATA.DAT for LBC
 *	
 *	Global Variables modified:	lbc_type, rbc_type, time_steps, t, dt, dx, 
 *								nodes, T[], conduct[], heatcap[], density[],
 *								lbc1[], lbc2[], rbc1[], rbc2[];
 *
 *
 * ------------------------------------------------------------------------- */

int GetData(void) 
{
	int g, h, i, j, k, m, n, loc, err, inp, val;
	char property[20], junk[85], *pprop;
	double targD, targdist;
	FILE *fpdata, *fpinter, *fpimpjet;

	i = 0;

/* open the datafile with ptr fpdata or return 0 to main if file doesn't exist */
	sprintf (junk, "%sDATA.DAT", datadir);
	if ((fpdata=fopen(junk, "rb")) == NULL) return (0); 

/* discard all file header lines, all should begin with a '#' */
	for (i=1; ;i++) 								
	{
		fgets (junk, 80, fpdata);					/* read 80 characters = 1 line of input file */
		if (junk[0] != '#') break;					
	}

/* read the output controls */
	fscanf(fpdata, "%*s %*s %*s %*s %*s %*s %*s %s %*s", junk); if (strlen(junk)) printtime = atof(junk);	/* time for T vs. depth  */
	fscanf(fpdata, "%*s %*s %*s %*s %*s %*s %s %*s", junk);		if (strlen(junk)) printnode = atoi(junk);/* node for T vs. time*/
	fscanf(fpdata, "%*s %*s %*s %*s %*s %s", junk);			if (strlen(junk)) VERBOSE = atoi(junk);	/* Output to File(1) or Screen(0) */
	fscanf(fpdata, "%*s %*s %s", junk);							if (strlen(junk)) printA = atoi(junk);	/* request A-matrix output*/
	fscanf(fpdata, "%*s %*s %s", junk);							if (strlen(junk)) printI = atoi(junk);	/* request Initial Node Temps */
	fscanf(fpdata, "%*s %*s %*s %*s %*s %*s %s", junk);			if (strlen(junk)) multio = atoi(junk);	/* request Node where Omega=1 */
	fscanf(fpdata, "%*s %*s %s", junk);							if (strlen(junk)) BATCH = atoi(junk);	/* request Initial Node Temps */

/* read the trial info. - Allows user to append mulitple runs in the same file yet decifer between the outputs */
	fscanf(fpdata, "%*s %*s %s", junk);		if (strlen(junk)) trial = atoi(junk);	/* read a user defined Trial number */
	fscanf(fpdata, "%*s %*s %*s %s", junk); if (strlen(junk)) strcpy(date, junk);	/* user entered Trial date */
	fscanf(fpdata, "%*s %*s %*s %s", junk); if (strlen(junk)) strcpy(inits, junk);	/* user initials */

/* read the burn integral constants */
	fscanf(fpdata, "%*s %*s %*s %s", junk); if (strlen(junk)) Gc = atof(junk);	/* G constant used to calculate Omega */
	if (Gc == 0.00) {Gc = 3.442707; 	expn = 94; }		/* if true: use default character and mantissa of G constant for Omega */
	else {
		fscanf(fpdata, "%*s %s", junk);				/* read a mantissa value from DATA.DAT if the user inputted a non-default value */
		if (strlen(junk)) {
			j = strlen(junk);							/* find the length of the string */
			for (i=0;i<(j-3);i++) junk[i] = junk[3+i];	/* read and save the values after the '10^' portion of the string */
			junk[j-3] = '\0';							/* terminate the string */
			expn = atoi(junk);							/* set expn equal to the integer value of the string */
		} 
	}
	fscanf(fpdata, "%*s %s", junk); if (strlen(junk)) dE = atof(junk);	/* read Delta-E constant */
	if (dE == 0.00) dE = 623580.00;										/* if input is 0, set dE to default value */
	fscanf(fpdata, "%*s %s", junk); if (strlen(junk)) R = atof(junk);	/* read R constant */
	if (R == 0.00) R = 8.31439;											/* if input value is 0.00, set R to default */
	
/* read the Left and Right Boundary Condition types */
	fscanf(fpdata, "%*s %*s %s", junk); if (strlen(junk)) strcpy(lbc_type, junk); 
	fscanf(fpdata, "%*s %s",  junk);     if (strlen(junk)) strcpy(rbc_type, junk);


/* read the time and depth data elements */	
	fscanf(fpdata, "%*s %*s %s", junk);     if (strlen(junk))  time_steps = atoi(junk);	/* total number of time steps for solver iterations */
	fscanf(fpdata, "%*s %s %*s", junk);		if (strlen(junk))      dt = atof(junk);	/* time increment used for each time step */
	fscanf(fpdata, "%*s %s %*s", junk);		if (strlen(junk))      dx = atof(junk);	/* distance increment between nodes */
	fscanf(fpdata, "%*s %s", junk);			if (strlen(junk))       nodes = atoi(junk);	/* total number of nodes in system, starting with 1 */

/* check for errors in user controls  */
	if (printtime < 0.00 || printtime > time_steps*dt) { 
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR1); 
		printf("%s", BLANKLINE); return (0);}
	if (printnode < 0 || printnode > nodes) {	
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR2); 
		printf("%s", BLANKLINE); return (0);}
	if (VERBOSE < 0 || VERBOSE > 1) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR3); 
		printf("%s", BLANKLINE); return (0);}
	if (printA < 0 || printA > 1) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR4); 
		printf("%s", BLANKLINE); return (0);}
	if (printI < 0 || printI > 1) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR5); 
		printf("%s", BLANKLINE); return (0);}
	if (multio < 0 || multio > 1) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR6); 
		printf("%s", BLANKLINE); return (0);}
	if (BATCH < 0 || BATCH > 1) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR7); 
		printf("%s", BLANKLINE); return (0);}
	if (multio == 1 && printtime != 0) {
		printf("%s", BLANKLINE); printf ("DATA.DAT Error: %s", DATAERROR8); 
		printf("%s", BLANKLINE); return (0);}
	

/* read and interpolate initial node temperatures  */
	inp = 0;
	fscanf(fpdata, "%*s %s", junk); if (strlen(junk)) strcpy(property, junk);			/* copies the junk value into property */
	fscanf(fpdata, "%s %*s", junk); if (strlen(junk))   j = atoi(junk);			/* j is number of data pairs for the Node Temperatures */
	if (j < 0) {		/* if true: output error message stating j must be at least equal to 0 */
		for (err=1;err<=23;err++) printf("%s", BLANKLINE);
		printf ("DATA.DAT Error: %s", DATAERROR9); 
		return (0); } 
	
	if (j == 0) {		/* if true:  open and read the initial temperature distribution from external file */
		sprintf (junk, "%sINITTEMP.DAT", datadir);
		if ((fpinter=fopen(junk, "rb")) == NULL) {
			printf("GetData() Error: %s", NOINITTEMP);	
			return (0); } 
		for (h=1; ;h++) {								
			fgets (junk, 80, fpinter);					/* read 80 characters = 1 line of input file */
			if (junk[0] != '#') break;					
		}
		fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			/* k is number of data pairs in the external file */
		
		for (m=1; ;m++) {		/* read any number of data pairs listed - until a character is read (files have '!' flags) */
			fscanf(fpinter, "%s", junk); 
			if (isdigit(junk[0]) || junk[0] == '.')   T[m].x = atof(junk);	/* T[].x is the node depth */
			else break;
			fscanf(fpinter, "%s", junk); if (strlen(junk))   T[m].y = atof(junk);	/* T[].y is the node Temperature */
			inp +=1;		/* increment the number of pairs read and verify with k */
		}
		fclose (fpinter);		/* close file pointer fpinter */
		j = k;		/* set j equal to k for Interp() call below */
		fscanf(fpdata, "%s", junk);		/* read the '!' at the beginning of the next line in DATA.DAT once fpinter is closed */
		if (junk[0] != '!') 
		{ 
			printf("Initial Temperature Distribution Error: \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
			(void) getchar();
			exit(0); 
		}  
	}
	else for (m=1; ;m++) {		/* if using data in DATA.DAT read all data pairs up to the next character */
		fscanf(fpdata, "%s", junk); 
		if (isdigit(junk[0]) || junk[0] == '.') T[m].x = atof(junk);
		else break;
		fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') T[m].y = atof(junk);	
		inp += 1;
	}	
	if (j > 0 && inp != j) 
	{
		printf("Initial Temperature Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
		printf("%s", DATACHECK1);
		(void) getchar();
		j = inp;
		for (err=1;err<=23;err++) printf("%s", BLANKLINE);
	}
	
	
/* call Interp() */
	pprop = &property[0];		/* set pointer equal to initial character of current property (i.e. T - used for switch in Inter() */
	Interp(pprop, j);

/* read the remainder of the possible data sets */
	for (i=0;i<7;i++) { 
		fscanf(fpdata, "%s", junk); if (strlen(junk)) strcpy(property, junk);	/* read Boundary Condition Type*/
		fscanf(fpdata, "%s %*s", junk); if (strlen(junk))   j = atoi(junk);		/* get number of data groups, ignore units */
		if (j < -1) {
			for (err=1;err<=23;err++) printf("%s", BLANKLINE);
			printf ("DATA.DAT '%s' Error: %s", property, DATAERROR10); 
			return (0); 
		} 
	
		if (property[0] == 'L' && property[5] == 'A') strcpy(property, "O");	/* change the identifier for the second LBC input */
		if (property[0] == 'R' && property[6] == 'A') strcpy(property, "A");	/* change for second RBC input - used in switch for interp */
	
/* switch, based on the first letter of the property name, to the appropriate case for reading values  - 
	allows the user to input the material properties and boundacry condition values in any order and allows
	them to omit sets if they are known to be irrelevant */
		switch(property[0]) {
			case 'C': {
				inp = 0;
				if (j == 0) {
					sprintf (junk, "%sCONDUCT.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData Error: %s", NOCONDUCT); return (0); }		
					for (h=1; ;h++) {	fgets (junk, 80, fpinter); if (junk[0] != '#') break; }
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			/* j is number of data pairs for the Node Temperatures */
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   conduct[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   conduct[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Conductivity Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') conduct[m].x = atof(junk);
					else  break; 
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') conduct[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
				{
					printf("Conductivity Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				}
				

				pprop = &property[0];		
				Interp(pprop, j);			
					}
				break;
			case 'H': {
				inp = 0;
				if (j == 0) {
					sprintf (junk, "%sHEATCAP.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("%s", NOHEATCAP); return (0); } 
					for (h=1; ;h++) {	fgets (junk, 80, fpinter);	if (junk[0] != '#') break;	}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			

					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   heatcap[m].x = atof(junk);	
						else break; 
						fscanf(fpinter, "%s", junk); if (strlen(junk))   heatcap[m].y = atof(junk);	
						inp +=1;
					}	
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Heat Capacity Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') heatcap[m].x = atof(junk);
					else  break; 
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') heatcap[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
				{
					printf("Heat Capacity Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				}
				
				pprop = &property[0];		
				Interp(pprop, j);			
					}
				break;
			case 'D': {
				inp = 0;
				if (j == 0) {
					sprintf (junk, "%sDENSITY.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData() Error: %s", NODENSITY); return (0); } 
					for (h=1; ;h++) { fgets (junk, 80, fpinter); if (junk[0] != '#') break;	}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   density[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   density[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Density Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') density[m].x = atof(junk);
					else break; 
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') density[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
				{
					printf("Density Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				}
				

				pprop = &property[0];
				Interp(pprop, j);
					}
				break;
			case 'L': {
				inp = 0;
				if (j == 0) {		
					sprintf (junk, "%sLEFTBC.DAT", datadir);  
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData() Error: %s", NOLBC); return (0); } 
					for (h=1; ;h++) { 
						fgets (junk, 80, fpinter); 
						if (junk[0] != '#')  break;	
					}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   lbc1[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   lbc1[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);		
					if (junk[0] != '!') 
					{ 
						printf("Left Boundary Condition Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else if (j == -1) {		/* if true, use of the Impinging Jet Convection Model */
					jet = 1;		/* set flag to signal use of impinging jet model */
					sprintf (junk, "%sLEFTIMP.DAT", datadir);		/* open data file LEFTIMP.DAT */
					if ((fpimpjet=fopen(junk, "rb")) == NULL) {	printf("GetData() Error: %s", NOLEFTIMP); return (0); } 
					for (h=1; ;h++) { fgets (junk, 80, fpimpjet); if (junk[0] != '#') break; }
	
				/* read data from LEFTIMP data file */
					fscanf(fpimpjet, "%*s %*s %*s %s", junk); if (strlen(junk))   onaxisbump = atof(junk);	/*singal the use of the bump term in the on-axis calculation */
					fscanf(fpimpjet, "%*s %*s %s", junk); if (strlen(junk))   Reknown = atof(junk);			/* Reynolds number if known and constant, if 0 use other input to calculate */
					fscanf(fpimpjet, "%*s %*s %s", junk); if (strlen(junk))   ventD = atof(junk);			/* jet vent diameter */
					fscanf(fpimpjet, "%*s %*s %s", junk); if (strlen(junk))   targD = atof(junk);			/* target diameter */
					fscanf(fpimpjet, "%*s %*s %s", junk); if (strlen(junk))   targdist = atof(junk);		/* distance to skin from jet outlet */
					fscanf(fpimpjet, "%*s %*s %s %*s", junk); if (strlen(junk))  k = atoi(junk);	/* number of LBC input pairs */
					val = k;		/* set val equal to k for comparison to inp */
					for (n=1; ;n++) {		/* the LBC datapairs set the duration of the convection input */
						fscanf(fpimpjet, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   lbc1[n].x = atof(junk);	
						else break; 
						fscanf(fpimpjet, "%s", junk); if (strlen(junk))   lbc1[n].y = atof(junk);	
						inp += 1;
					}
					if (inp != val)		/* if true:  output warming to user and wait for response */
					{
						printf("Left Boundary Condition Distribution (LEFTIMP.DAT): %s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, val);
						printf("%s", DATACHECK1);
						(void) getchar();
						k = val;		/* if user accepts to continue reset k to val */
						for (err=1;err<=23;err++) printf("%s", BLANKLINE);
						inp = 0;
					}
					inp = 0;		/* reset inp for next loop of data pairs */
					fscanf(fpimpjet, "%*s %s %*s", junk); if (strlen(junk))  g = atoi(junk);		/* number of time, gas-velocity time pairs */
					for (m=1; ;m++) {
						fscanf(fpimpjet, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   gasV[m].x = atof(junk);	
						else break;
						fscanf(fpimpjet, "%s", junk); if (strlen(junk))   gasV[m].y = atof(junk);	
						inp += 1;
					}
					fclose (fpimpjet);
					if (inp != g)		/* output warning of mismatched data */
					{
						printf("Gas Velocity Distribution (LEFTIMP.DAT): %s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, g);
						printf("%s", DATACHECK1);
						(void) getchar();
						inp = 0;
					}
	
					onaxis = targdist/ventD;		/* calculate the onaxis spacing */
					offaxis = targD/2/ventD;		/* calculate the offaxis spacing */
	
					if (offaxis < 0.00)  {		/* if true: user input invalid number for offaxis spacing */
						printf("\r\nGetData() Error:  %s", OFFERROR1);
						printf("%s", DATACHECK1);
						return 0;
					}
					j = k;  /* j has to be set back to a number of sets so that the Interp() can be done */
					fscanf(fpdata, "%s", junk); 
					if (junk[0] != '!') 
					{ 
						printf("Left Boundary Condition Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					loc = 1;
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') lbc1[m].x = atof(junk);
					else break; 
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') lbc1[m].y = atof(junk);	
					inp += 1;
				}	
				if (loc == 1 && inp != j) 
				{
					printf("Left Boundary Condition Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				} 
				

				pprop = &property[0];
				Interp(pprop, j);
					}
				break;
			case 'O': {
				inp = 0;
				if (j == 0) {
					sprintf (junk, "%sLEFTAMB.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData() Error: %s"); return (0); } 
					for (h=1; ;h++) { fgets (junk, 80, fpinter); if (junk[0] != '#') break;	}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   lbc2[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   lbc2[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Left-side Ambient Temperature Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') lbc2[m].x = atof(junk);
					else break;
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') lbc2[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
				{
					printf("Left-side Ambient Temperature Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				}
					
				if (jet) dur = lbc2[2].x;
	
				pprop = &property[0];
				Interp(pprop, j);
					}
				break;
			case 'R': {
				inp = 0;
				if (j == 0) {	
					sprintf (junk, "%sRIGHTBC.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData() Error: %s", NORBC); return (0); } 
					for (h=1; ;h++) { fgets (junk, 80, fpinter); if (junk[0] != '#') break;	}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   rbc1[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   rbc1[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Right Boundary Condition Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') rbc1[m].x = atof(junk);
					else break;
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') rbc1[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
					{
						printf("Right Boundary Condition Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
						printf("%s", DATACHECK1);
						(void) getchar();
						j = inp;
						for (err=1;err<=23;err++) printf("%s", BLANKLINE);
					}
				

				pprop = &property[0];
				Interp(pprop, j);
					}
				break;
			case 'A': {
				inp = 0;
				if (j == 0) {
					sprintf (junk, "%sRIGHTAMB.DAT", datadir);
					if ((fpinter=fopen(junk, "rb")) == NULL) { printf("GetData() Error: %s", NORIGHTAMB); return (0); } 
					for (h=1; ;h++) { fgets (junk, 80, fpinter); if (junk[0] != '#') break;	}
					fscanf(fpinter, "%*s %*s %s", junk); if (strlen(junk))   k = atoi(junk);			
		
					for (m=1; ;m++) {
						fscanf(fpinter, "%s", junk); 
						if (isdigit(junk[0]) || junk[0] == '.')   rbc2[m].x = atof(junk);	
						else break;
						fscanf(fpinter, "%s", junk); if (strlen(junk))   rbc2[m].y = atof(junk);	
						inp +=1;
					}
					fclose (fpinter);
					j = k;
					fscanf(fpdata, "%s", junk);
					if (junk[0] != '!') 
					{ 
						printf("Right-side Ambient Temperature Distribution Error:  \r\n%s%s\r\n", DATAERROR11, DATACHECK3);
						(void) getchar();
						exit(0);
					}
				}
				else for (m=1; ;m++) {
					fscanf(fpdata, "%s", junk); 
					if (isdigit(junk[0]) || junk[0] == '.') rbc2[m].x = atof(junk);
					else break;
					fscanf(fpdata, "%s", junk); if (isdigit(junk[0]) || junk[0] == '.') rbc2[m].y = atof(junk);	
					inp += 1;
				}
				if (j > 0 && inp != j) 
				{
					printf("Right-side Ambient Temperature Distribution: \r\n%s (%d)\n%s (%d)\r\n", READERROR1, inp, READERROR2, j);
					printf("%s", DATACHECK1);
					(void) getchar();
					j = inp;
					for (err=1;err<=23;err++) printf("%s", BLANKLINE);
				}
				

				pprop = &property[0];
				Interp(pprop, j);
					}
				break;		
			default: 
				break;
		}
	}
	
	fclose (fpdata);						/* close the pointer to the data file */
	
return 1;								/* return 1 after successful completion */
	
}	/* End of GetData()  */

/* ---------------------------------------------------------------------*
 *
 *	Function Name:  double GenInt(double a, double b, double c, double d, double e)
 *
 *	Purpose:  Interpolates and returns the value 'intnum' from the remainder of the input values
 *
 *	Arguments:	a, b, c, d, e - five values necessary for interpolation
 *			Interpolation format
 *							LHS		|	RHS
 *							a		|	d
 *							b		|	intnum
 *							c		|	e
 *
 *	Return:  intnum - returns the interpolated value as a double
 *	
 *	Created:	7/17/98	SJR
 *	Modified:			Modification: 
 *	Modified:			Modification:	
 *
 *	Local Variables:	double intnum - the value that is interpolated and returned to calling function
 *						double num1, num2, den:  temporary values in calculation of intnum
 *
 *	Global Variables modified:	none
 *
 * ------------------------------------------------------------------------- */


double GenInt(double a, double b, double c, double d, double e) {

double intnum, num1, num2, den;

			num1 = (a - b);			
			num2 = (d - e);
			den  = (a - c) ;
			intnum = d - (num1*num2)/den; 
			

return(intnum);

}	/* End of GetInt()

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  double JetConvection(int incr, double temp, double amb)
 *	Purpose:  Calculates the Average Convection coefficient (h-bar) for every time step
 *
 *	Arguments:			int incr - time_steps is passed with the function as incr
 *						double temp - Temperature of skin surface   T[1].y
 *						double amb -  Temperature of the Jet Outlet RBC2[].y
 *	Return:  1 if successful 
 *	
 *	Created:	7/13/98	SJR
 *	Modified:	7/23/98	Modification: added argument double amb because Tout is LBC2
 *	Modified:			Modification:	
 *
 *	Local Variables:	int i -	used for loops
 *						int j - used for loops
 *						char junk[85] - used for placing the filename into the FILE pointer
 *						double Tfilm - calculated average of inputs 'temp' and 'amb'
 *						double Pr - current temperature based gas Prandtl number
 *						double kN - current temperature based gas Conductivity
 *						double Velocity - interpolated gas velocity used per time step to find Reynolds number
 *						double Viscosity - interpolated gas kinematic viscosity used per time step to find Reynolds number
 *						double Nu - Reynolds based Nusselt Number
 *						double Nustag - Nusselt number at Stagnation point
 *						double NuOn - On-axis Nusselt number
 *						double NuOff - Off-axis Nusselt number
 *						double Nubar - product of Average Nusselt Number and Off-axis Nusselt Number
 *						double Nuavg - average Nusselt number over 100 evenly calculated steps
 *						double Nutemp - temporary storage of Nusselt number for calculations
 *						double aOff - exponential variable used in Off-axis calculation
 *						double cOff - denominator variable used in Off-axis calculations
 *						double BetaOff - variable used in Off-axis calculation
 *						double rdstep - incremental variable used in Off-axis calculation
 *						double NuDist[50] - array used to integrate the function over 100 steps
 *						struct datapair Visknown[50] - array used to hold Temperature, Kinematic Viscosity data pairs
 *						struct datapair Prknown[50] - array used to hold Temperature, Prandtl number data pairs
 *						struct datapair Known[50] - array used to hold Temperature, Conductivity (k) data pairs
 *	
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */

double JetConvection(int incr, double temp, double amb) {			
int i, j;
char junk[85];
double Tfilm, Pr, kN, Velocity, Viscosity, Re;
double Nu, Nustag, NuOn, NuOff, Nubar, Nuavg, NuDist[50], Nutemp; 
double aOff, cOff, BetaOff, rdstep;
struct datapair Visknown[50];
struct datapair Prknown[50];
struct datapair Known[50];
FILE *fpgas;

/* Read the gas properties for the impinging jet model: allows for any gas to be used */
sprintf (junk, "%sGASPROP.DAT", datadir);
if ((fpgas=fopen(junk, "rb")) == NULL) {
	printf("JetConvection() Error: %s", NOGASPROP);	
	return (0); } 
for (i=1; ;i++) { fgets (junk, 80, fpgas); if (junk[0] != '#') break; }
for (i=1; i<19;i++) {
	fscanf(fpgas, "%s", junk); if (strlen(junk)) { Visknown[i].x = atof(junk);  Known[i].x = atof(junk); Prknown[i].x = atof(junk);  }
	fscanf(fpgas, "%s", junk); if (strlen(junk))   Visknown[i].y = atof(junk);
	fscanf(fpgas, "%s", junk); if (strlen(junk))   Known[i].y = atof(junk);
	fscanf(fpgas, "%s", junk); if (strlen(junk))   Prknown[i].y = atof(junk);
}
fclose (fpgas);

/* If Reynolds number is inputted as 0.000, calculate the value based on Gas Velocity and Kinematic Viscosity input */
if (Reknown == 0.0000) {
 /* Interpolate for the gas velocity at the current time step */
 for (j=1; j< 1000 ; j++) {
	if (incr*dt >= gasV[j].x && incr*dt <= gasV[j+1].x)	{
		Velocity = GenInt(gasV[j+1].x, incr*dt, gasV[j].x, gasV[j+1].y, gasV[j].y);  
		j = 1000;  /* exit from loop once Velocity has been calculated */
	}
	else continue;				/* continue looping through gasV input until the 'if' conditions are met */
 }
 /* Interpolate for the kinematic viscosity based on the values from GASPROP.DAT  */
 for (j=1; j<18; j++) {	
	if (amb < Visknown[1].x) {		
		Viscosity = Visknown[1].y;
		i = 19;								/* exit from for() loop and return */
	}
	else if (amb > Visknown[18].x) {
		Viscosity = Visknown[18].y;
		i = 19;								/* exit from for() loop and return */
	}
	else if (amb >= Visknown[j].x && amb <= Visknown[j+1].x) {
		Viscosity = GenInt(Visknown[j+1].x, amb, Visknown[j].x, Visknown[j+1].y, Visknown[j].y);
		j = 19;
	}
	else continue;
 }

 Re = Velocity * ventD / Viscosity;			/* time_step and temperature dependent calculation of Reynolds number */
} 
else Re = Reknown;							/* set Reynolds number equal to the value inputted from IMPJET.DAT  */

rdstep = 0;									/* initialize rdstep for later use */

Tfilm = ((amb + temp)/ 2);					/* calculate the film temp. as the average of jet outlet temp and surface temp */

/* find the interpolated value of Pr */
for (i=1;i<=18;i++) {					/* loop through all stored values of Tcritdepth */
	if (Tfilm < Prknown[1].x) {			/* if film temp is below the lowest available known temp set Pr equal to lowest known */
		Pr = Prknown[1].y;
		i = 19;							/* exit from for() loop once Pr has been calculated */
	}
	else if (Tfilm > Prknown[18].x) {	/* if film temp is greater than greatest inputted film temp set Pr equal to highest */
		Pr = Prknown[18].y;	
		i = 19;					
	}
	else if (Tfilm >= Prknown[i].x && Tfilm <= Prknown[i+1].x) {	/* if film temp falls between any of the inputted temp. find Pr */
		Pr = GenInt(Prknown[i+1].x, Tfilm, Prknown[i].x, Prknown[i+1].y, Prknown[i].y);
		i = 19;
	}
	else continue;		/* if none of these conditions exist, continue with for() loop */
}

Nu = 1.1613 * pow(Re, .5041) * pow(Pr, .42);		/* Calculate the Reynolds-based Nusselt number */

/* On-axis calculation */
if (onaxis > 0.000000 && onaxis <= 5.5000000) {		/* calculate on-axis Nusselt number based on ratio of r/d */
	if (onaxisbump == 1) NuOn = 1.06 - .030 * onaxis + 2 * pow(10, -6) * Re * exp(-(pow(onaxis-4.4, 2)/.7));
	else NuOn = 1.06 - .030 * onaxis;				/* use this equation if bump term is not to be used */
}
else if (onaxis > 5.500000 && onaxis <= 10.000000) NuOn = 1.593 - 0.1266 * onaxis;
else if (onaxis > 10.000000 && onaxis <= 20.000000) NuOn = 0.6295 - 0.0287 * onaxis;
else {printf("\r\n\r\nError JetConvection():  %s", ONERROR1);  (void) getchar();  return 0; }

Nustag = Nu * NuOn;							/* calculate the Stagnation point Nusselt number */

Nutemp = Nustag;							/* put stagnation poitn Nu into a temporary variable */

/* Off-axis calculation */
if (offaxis > 0.000) {
	aOff = 2.4902 - 0.2330 * onaxis + 0.01462 * pow(onaxis, 2) + 6.2769 * pow(10, -6) * Re + 3.5356 * pow(10, -6) * Re * onaxis;

	if (Re >= 25000) cOff = 1.4 + Re * pow(10, -7) + 1.55 * pow(10, -4) * Re * exp(-(pow(onaxis, 2)/9));
	else cOff = 1.3 + Re * pow(10, -7) + 2.1710 * pow(10, -5) * Re * exp(-(pow(onaxis, 2)/9));

	if (Re <= 30000) BetaOff = 1.5139 - 1.624 * pow(10, -5) * Re;
	else BetaOff = 1.0;

	for (i=0;i<100;i++) {
		rdstep = rdstep + ((offaxis/2)/100);
		NuDist[i] = exp(-pow(onaxis/aOff, cOff)) + 5.2587 * pow(10, -4) + (2.6503 * pow(10, -2) * exp(-6.25 * pow(-BetaOff+rdstep, 2))/pow(onaxis, 2));
		Nuavg = Nuavg + NuDist[i];
	}
	NuOff = Nuavg/ 100;

	Nubar = Nutemp * NuOff;
}
else Nubar = Nutemp;

/* find the interpolated value of k at Tfilm */
for (i=1;i<=18;i++) {				/* loop through all stored values of Known*/
	if (Tfilm <= Known[1].x) {
		kN = Known[1].y;
		i = 19;					
	}
	else if (Tfilm >= Known[18].x) {
		kN = Known[18].y;
		i = 19;					
	}
	else if (Tfilm >= Known[i].x && Tfilm <= Known[i+1].x) {
		kN = GenInt(Known[i+1].x, Tfilm, Known[i].x, Known[i+1].y, Known[i].y);
		i = 19;
	}
	else continue;		/* if none of these conditions exist, continue with for() loop */
}

hbar = Nubar * kN / ventD;

return hbar;


}  /* End of JetConvection()  */


/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void Interp(char *pp, int sets)
 *	Purpose:	Interpolates all inputs from GetData() over depth or time
 *
 *	Arguments:	*pp - first letter of GetData.property; sets - # of data sets
 *	Return:  Void
 *	
 *	Created:	4/1/98	SJR
 *	Modified:			Modification: Condensed code
 *	Modified:			Modification:	
 *
 *	Local Variables:	int i, j, k;  - used for loops
 *						char junk[85] - temporary string storage
 *						double num1	  - haf of Interp() equation numerator 
 *						double num2	  - second haf of numerator
 *						double den	  - denominator
 *						struct datapair temp[MAXARRAY] - temporary storage
 *	
 *	Global Variables modified:	T[], conduct[], heatcap[], density[],
 *								lbc1[], lbc2[], rbc1[], rbc2[];
 *
 * ------------------------------------------------------------------------- */

void Interp(char *pp, int sets) {		
int i, j, k;
char junk[85];
struct datapair temp[MAXARRAY];
		

strcpy(junk, pp);

  switch(junk[0]) {
	case 'I': {
		for (i=1;i<=sets;i++) {	temp[i].x = T[i].x;	temp[i].y = T[i].y;	} /* copy input data pair into temporary */
		
		if (temp[sets].x < (nodes * dx))		/* if true: report error that the input data file has insufficient data */
		{
			printf("Datafile Error:  %s  %s", INTERPERR1, INTERPCHK1);
			(void) getchar();
			exit(0);
		}

		/* there are always two data pairs provided as inputs: 1st node and any node past the depth of the model */
		T[1].x = temp[1].x;					T[1].y = temp[1].y; 	/* first node value equal to the first temp value */
		
		for (j=2;j<=nodes;j++) {		/* for all Nodes, run through all Sets of temporary data */
			T[j].x = (j-1) * dx;
			for (k=1;k<=sets;k++)	{	
			  if (T[j].x > temp[k].x && T[j].x < temp[k+1].x) 	/* Check order of T[].x, if between, calculate the node.y value   */
			     T[j].y = GenInt(temp[k].x, T[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			  else if (T[j].x == temp[k].x) {			/* This condition covers the multiple input parameters (> two data pairs) */
				 T[j].y = temp[k].y;					/* If an inputted .x value matches a temporary.x value, set T[].y = temp .y */
				 k = sets+1;  }							/* After this occurs the model should stop looping through the sets */
			  else continue;
			}
		}
			  }
		break;
	case 'C': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = conduct[i].x; temp[i].y = conduct[i].y; }

		if (temp[sets].x < (nodes * dx)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR2, INTERPCHK1);
			(void) getchar();
			exit(0);
		}

		conduct[1].x = temp[1].x;			conduct[1].y = temp[1].y;	
		
		for (i=2;i<=nodes-1;i++) { conduct[i].x = conduct[i-1].x + dx; conduct[i].y = temp[1].x; } 

		for (j=2;j<=nodes;j++) {
		  conduct[j].x = (j-1) * dx;
		  for (k=1;k<=sets;k++)	{
			if (conduct[j].x > temp[k].x && conduct[j].x < temp[k+1].x) 
				conduct[j].y = GenInt(temp[k].x, conduct[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (conduct[j].x == temp[k].x) { conduct[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	case 'H': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = heatcap[i].x; temp[i].y = heatcap[i].y; }
	
		if (temp[sets].x < (nodes * dx)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR3, INTERPCHK1);
			(void) getchar();
			exit(0);
		}

		heatcap[1].x = temp[1].x;			heatcap[1].y = temp[1].y; 
		 
		for (i=2;i<=nodes-1;i++) { heatcap[i].x = heatcap[i-1].x + dx; heatcap[i].y = temp[0].x; } 

		for (j=2;j<=nodes;j++) {
		  heatcap[j].x = (j-1) * dx;
		  for (k=1;k<=sets;k++)	{
			if (heatcap[j].x > temp[k].x && heatcap[j].x < temp[k+1].x)
				heatcap[j].y = GenInt(temp[k].x, heatcap[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (heatcap[j].x == temp[k].x) { heatcap[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	case 'D': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = density[i].x; temp[i].y = density[i].y; }

		if (temp[sets].x < (nodes * dx)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR4, INTERPCHK1);
			(void) getchar();
			exit(0);
		}

		density[1].x = temp[1].x;			density[1].y = temp[1].y;
		
		for (i=2;i<=nodes-1;i++) { density[i].x = density[i-1].x + dx; density[i].y = temp[1].x; }

		for (j=2;j<=nodes;j++) {
		  density[j].x = (j-1) * dx;
		  for (k=1;k<=sets;k++)	{
			if (density[j].x > temp[k].x && density[j].x < temp[k+1].x) 
				density[j].y = GenInt(temp[k].x, density[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (density[j].x == temp[k].x) { density[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			 }
		break;
	case 'L': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = lbc1[i].x; temp[i].y = lbc1[i].y; }
		
		if (temp[sets].x < (time_steps * dt)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR5, INTERPCHK2);
			(void) getchar();
			exit(0);
		}

		lbc1[1].x = temp[1].x;				lbc1[1].y = temp[1].y;
		
		for (i=2;i<=time_steps-1;i++) { lbc1[i].x = lbc1[i-1].x + dt; lbc1[i].y = temp[1].x; }  

		for (j=2;j<=time_steps+1;j++) {
		  lbc1[j].x = (j-1) * dt;
		  for (k=0;k<=sets;k++)	{
			if (lbc1[j].x > temp[k].x && lbc1[j].x < temp[k+1].x) 
				lbc1[j].y = GenInt(temp[k].x, lbc1[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (lbc1[j].x == temp[k].x) { lbc1[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	case 'O': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = lbc2[i].x; temp[i].y = lbc2[i].y; }

		if (temp[sets].x < (time_steps * dt)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR6, INTERPCHK2);
			(void) getchar();
			exit(0);
		}

		lbc2[1].x = temp[1].x;				lbc2[1].y = temp[1].y;
		
		for (i=2;i<=time_steps-1;i++) { lbc2[i].x = lbc2[i-1].x + dt; lbc2[i].y = temp[1].x; }

		for (j=2;j<=time_steps+1;j++) {
		  lbc2[j].x = (j-1) * dt;
		  for (k=1;k<=sets;k++)	{
			if (lbc2[j].x > temp[k].x && lbc2[j].x < temp[k+1].x) 
				lbc2[j].y = GenInt(temp[k].x, lbc2[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (lbc2[j].x == temp[k].x) { lbc2[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	case 'R': {			/* refer to case T for documentation */
		for (i=1;i<=sets;i++) { temp[i].x = rbc1[i].x; temp[i].y = rbc1[i].y; }

		if (temp[sets].x < (time_steps * dt)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR7, INTERPCHK2);
			(void) getchar();
			exit(0);
		}

		rbc1[1].x = temp[1].x;				rbc1[1].y = temp[1].y;
		
		for (i=2;i<=time_steps-1;i++) {	rbc1[i].x = rbc1[i-1].x + dt; rbc1[i].y = temp[1].x; }

		for (j=2;j<=time_steps+1;j++) {
		  rbc1[j].x = (j-1) * dt;
		  for (k=1;k<=sets;k++)	{
			if (rbc1[j].x > temp[k].x && rbc1[j].x < temp[k+1].x) 
				rbc1[j].y = GenInt(temp[k].x, rbc1[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (rbc1[j].x == temp[k].x) { rbc1[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	case 'A': {			/* refer to case T for documentation.  This contains the printf statements used for debugging  */
		for (i=1;i<=sets;i++) { temp[i].x = rbc2[i].x; temp[i].y = rbc2[i].y; }

		if (temp[sets].x < (time_steps * dt)) 
		{
			printf("Datafile Error:  %s  %s", INTERPERR8, INTERPCHK2);
			(void) getchar();
			exit(0);
		}

		rbc2[1].x = temp[1].x;				rbc2[1].y = temp[1].y;
		
		for (i=2;i<=time_steps-1;i++) {	rbc2[i].x = rbc2[i-1].x + dt; rbc2[i].y = temp[1].x; } 

		for (j=2;j<=time_steps+1;j++) {
		  rbc2[j].x = (j-1) * dt;
		  for (k=1;k<=sets;k++)	{
		  	if (rbc2[j].x > temp[k].x && rbc2[j].x < temp[k+1].x) 
				rbc2[j].y = GenInt(temp[k].x, rbc2[j].x, temp[k+1].x, temp[k].y, temp[k+1].y);
			else if (rbc2[j].x == temp[k].x) { rbc2[j].y = temp[k].y; k = sets+1; }
			else continue;
		  }
		}
			}
		break;
	default: {printf("Interp() Error:  %s", INTERROR1); 
		printf("%s", INTCHECK1);
		(void) getchar(); }
		break;
  }
			
return;

}		/* End of Interp() */


/* ----------------------------------------------------------------------------
 *
 *	Function Name:  int Fourier()
 *	Purpose:  Calculates the Fourier (Fo) number for each node 
 *
 *	Arguments:	None
 *	Return:  1 if successful 
 *	
 *	Created:	4/1/98	SJR
 *	Modified:			Modification: 
 *	Modified:			Modification:	
 *
 *	Local Variables:	int i
 *						double num - numerator of Fourier equation
 *						double den -	denominator of Fourier equation
 *	
 *	Global Variables modified:	fo[]
 *
 * ------------------------------------------------------------------------- */

int Fourier() {			
int i;
double num, den;

num = 0.000000000000;
den = 0.000000000000;

	for (i=1;i<=nodes;i++) {
		num = conduct[i].y * dt;
		den = (density[i].y * dx)* (heatcap[i].y * dx);
		fo[i] = num/den;
	}

return 1;  

}		/* End of Fourier() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  int InterNodes(void)
 *	Purpose: Calculates the A-matrix values for the interior nodes
 *
 *	Arguments:	None
 *	Return:  1 if successful
 *	
 *	Created:	4/1/98  SJR
 *	Modified:			Modification: 
 *	Modified:			Modification:	
 *
 *	Local Variables:	int i
 *						double a - used to calculate the average k for To @ time p+1 
 *						double b - used to calculate the average k for T1 @ time p+1
 *	
 *	Global Variables modified:	lower[], diag[], upper[]
 *
 * ------------------------------------------------------------------------- */


int InterNodes(void) {		
int i;					
double a, b;
	/* These formulae are always the same because interior nodes are always Conductive */
	for (i=2;i<=nodes-1;i++) {
		a = ((two * conduct[i-1].y)/(conduct[i].y + conduct[i-1].y));
		b = ((two * conduct[i+1].y)/(conduct[i].y + conduct[i+1].y));

		lower[i] = (negone * a) * fo[i];
		diag[i]  = (one + (a * fo[i]) + (b * fo[i]));
		upper[i] = (negone * b) * fo[i];
	}

return 1;  

}		/* End of InterNodes() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void Convection(char bc, int step)
 *	Purpose: Calculates the L- and/or R- Convection A-Matrix values
 *
 *	Arguments:	char bc - L (left) or R (right)
 *				int step - the time step at which the values are calculated
 *	Return:  void
 *	
 *	Created:	4/1/98	SJR
 *	Modified:	8/10/98	Modification: Added Impinging Jet functionality for left side only
 *	Modified:			Modification:	
 *
 *	Local Variables:	double a - a portion of the coefficient for To
 *						double b - constant used to calculate a
 *	
 *	Global Variables modified:	bifo[], lower[], diag[], upper[], rhs[];
 *
 * ------------------------------------------------------------------------- */

void Convection(char bc, int step) {	/* takes an L or R from and the current time step integer */	
double a;								


	if (bc == 'L') {	/* Left side convection equation */
		a = (four * conduct[2].y) / (conduct[1].y + conduct[2].y);	/* first coefficient is based on the average k between the two nodes */
		if (jet == 1 && lbc1[step].y == 1)
			bifo[1] = ((jetdur[step].y * (dt/dx) )/ (density[1].y * heatcap[1].y));	/* BiFo is based on the condensed value of Bi and Fo equations */
		else
			bifo[1] = ((lbc1[step].y * (dt/dx) )/ (density[1].y * heatcap[1].y));	/* BiFo is based on the condensed value of Bi and Fo equations */
		
		diag[1] = (one + (a * fo[1]) + (two * bifo[1]));	/* T(o) coefficient at time p+1 */
		upper[1] = (negone*a) * fo[1];					/* T(1) coefficient at time p+1 */

		rhs[1] = T[1].y + (two * bifo[1] * lbc2[step].y);	/* RHS-Matrix value for row one at time p */
	}
	else if (bc == 'R') {		/* same as for L except coefficient values change */
		a = (four * conduct[nodes-1].y) / (conduct[nodes-1].y + conduct[nodes].y);
		bifo[nodes] = ((rbc1[step].y * (dt/dx)) / (density[nodes].y * heatcap[nodes].y));
		
		lower[nodes] = (negone*a) * fo[nodes];
		diag[nodes] = (one + (a * fo[nodes]) + (two * bifo[nodes]));
		rhs[nodes] = T[nodes].y + (two * bifo[nodes] * rbc2[step].y);
	}
	else {printf("Convection() Error: %s", CONVECTERR);  (void) getchar();
	}

}		/* End of Convection() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void Flux(char bc, int step)
 *	Purpose: Calculates the L- and/or R-Flux A-Matrix values
 *
 *	Arguments:	char bc - L (left) or R (right)
 *				int step - the time step at which the values are calculated
 *	Return:  void
 *	
 *	Created:	4/1/98	SJR
 *	Modified:			Modification: moved local variable to #define as 'four'
 *	Modified:			Modification:	
 *
 *	Local Variables:	double a - a portion of the coefficient for To
 *	
 *	Global Variables modified:	lower[], diag[], upper[], rhs[];
 *
 * ------------------------------------------------------------------------- */

void Flux(char bc, int step) 
{		
	double a;

	if (bc == 'L') {	
		a = four * conduct[2].y / (conduct[1].y + conduct[2].y);
				
		diag[1] = (one + a * fo[1]);
		upper[1] = (negone*a) * fo[1];
		rhs[1] = T[1].y + ((two * lbc1[step].y * (dt/dx)) / (heatcap[1].y * density[1].y)); 
	}
	else if (bc == 'R') 
	{
		a = four * conduct[nodes-1].y / (conduct[nodes-1].y + conduct[nodes].y);
				
		lower[nodes] = (negone*a) * fo[nodes];
		diag[nodes] = (one + a * fo[nodes]);
		rhs[nodes] = T[nodes].y - ((two*rbc1[step].y*(dt/dx))/(heatcap[nodes].y * density[nodes].y)); 
	}
	else 
	{
		printf("Flux() Error: %s", FLUXERROR);
		(void) getchar();
	}

}		/* End of Flux() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void InterRHS(void)
 *	Purpose: Calculate the RHS-Matrix for the interior nodes
 *	
 *	Arguments:	None
 *	Return:  void
 *	
 *	Created:	4/1/98	SJR
 *	Modified:			Modification: 
 *	Modified:			Modification:	
 *
 *	Local Variables:	int i - for loops
 *	
 *	Global Variables modified:	rhs[]
 *
 * ------------------------------------------------------------------------- */


void InterRHS(void) 
{		
	int i;

	for (i=2;i<=nodes-1;i++) rhs[i] = T[i].y;

}	/* End of InterRHS() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void nrerror(char error_text[])
 *	Purpose: Numerical Recipes standard error handler 
 *
 *	Arguments:	char error_text[] - 
 *
 *	Return:  void
 *	
 *	Created:	4/1/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	
 *	
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */


void nrerror(char error_text[]) {

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"Check that all input files and data are available...now exiting to system...\n");
	(void) getchar();
	exit(1);

}		/* End of nrerror() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  double *vector(long nl, long nh)
 *
 *	Purpose: allocate a double vector with subscript range v[nl..nh] 
 *
 *	Arguments:	long nl - 
 *				long nh - 
 *
 *	Return:  
 *	
 *	Created:	4/1/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	double *v - allocate memory to v
 *	
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */


double *vector(long nl, long nh) 
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("Allocation failure in vector(): check input data file for correct format.");
	return v-nl+NR_END;

}	/* End of *vector() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void free_vector(double *v, long nl, long nh)
 *	Purpose: Free a vector() from memory 
 *
 *	Arguments:	double *v -	pointer to memory
 *				long nl - size variable
 *				long nh - size variable
 *
 *	Return:  void
 *	
 *	Created:	4/1/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	
 *
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */


void free_vector(double *v, long nl, long nh) 
{

	free((FREE_ARG) (v+nl-NR_END));

}		/* End of free_vector() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void tridag(double a[], double b[], double c[], double r[], 
 *								double u[], unsigned long n)
 *	Purpose: Solve the tridiagonal system
 *	Arguments:	double a[] -
 *				double b[] -
 *				double c[] -
 *				double r[] -
 *				double u[] -
 *				unsigned long n -
 *	Return:  
 *	
 *	Created:	4/1/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	int i -
 *						unsigned long j -
 *						double bet -
 *						double *gam -
 *	
 *	Global Variables modified:	T[]
 *
 * ------------------------------------------------------------------------- */


void tridag(double a[], double b[], double c[], double r[], double u[], unsigned long n) 
{
	unsigned long j;
	double bet,*gam;
	int i;

	gam=vector(1,n);
	if (b[1] == 0.0) nrerror("tridag() Error: b[1] = 0.00");
	u[1]=r[1]/(bet=b[1]);		/* divide the first RHS-Matrix value by diag[1] */
	for (j=2;j<=n;j++) {
		gam[j]=c[j-1]/bet;		
		bet=b[j]-a[j]*gam[j];	
		if (bet == 0.0)	nrerror("tridag() Error: bet = 0.00");
		u[j]=(r[j]-a[j]*u[j-1])/bet;
	}
	for (j=(n-1);j>=1;j--)		/* solve for the u[] values that are the solution matrix */
		u[j] -= gam[j+1]*u[j+1];
	for (i=1;i<=nodes;i++)		/* set the solution matrix values equal to the T values for recompiling the RHS matrix for the next time step */
		T[i].y = u[i]; 


	free_vector(gam,1,n);		/* release the memory for the gam vector */


}		/* End of tridag() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  double **matrix(long nrl, long nrh, long ncl, long nch)
 *
 *	Purpose: Numerical Recipes utility used to build the matrix in memory
 *
 *	Arguments:	long nrl - 
 *				long nrh - 
 *				long ncl - 
 *				long nch - 
 *								
 *	Return:  double
 *	
 *	Created:	4/13/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	long i - 
 *						nrow, ncol -
 *						double **m - 
 *
 *	Global Variables modified:	none
 *
 * ------------------------------------------------------------------------- */


double **matrix(long nrl, long nrh, long ncl, long nch)  /* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void free_matrix(double **m, long nrl, long nrh, long ncl, long nch)
 *
 *	Purpose: NR utility used to free a double matrix allocated by matrix() 
 *
 *	Arguments:	double **m - 
 *				long nrl - 
 *				long nrh - 
 *				long ncl - 
 *				long nch - 
 *				void *derivs - see derivs() above
 *
 *	Return:  void
 *	
 *	Created:	4/13/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables: none 
 *
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */


void free_matrix(double **m, long nrl, long nrh, long ncl, long nch) {

	free((FREE_ARG) (m[nrl]+ncl-NR_END));
	free((FREE_ARG) (m+nrl-NR_END));

}		/* End of free_matrix() */


/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void derivs(double x,double y[],double dydx[])
 *
 *	Purpose: Calculate the derivatives of exp((dE_R/Tbar)+Beta) at the rk4 and rkdumb time steps
 *
 *	Arguments:	double x - represents the time at which the derivative is required
 *				double y[] - This is not used in the Burn Model because Omega is not a function of t AND Omega
 *				double dydx[] - the equated value of the derivative at a time step
 *
 *	Return:  void
 *	
 *	Created:	4/7/98	SJR based on Numerical Recipes (Copyright (C) 1986 - 1995) requirement
 *
 *	Local Variables:	i, k	- i for loops, k to reduce code line lengths
 *						struct datapair Tint[] - structure array used to find interpolated Temp. at not i*dt steps of x 
 *
 *	Global Variables modified:	dydx[]
 *
 * ------------------------------------------------------------------------- */

void derivs(double x,double y[],double dydx[]) 
{
	int i;
	struct datapair Tint[MAXARRAY];

/* Create dydx[1] for any value of x necessary */
	if (x == 0.00) dydx[1] = exp((-dE_R/Tcritdepth[0].y)+Beta);		/* dydx at time 0.00 */
	else 
	{										/* cover all other time values */
		for (i=1;i<=time_steps;i++) 				/* loop through all stored values of Tcritdepth */
		{
			if (x == Tcritdepth[i].x) 				/* if x equals a value stored in Tcritdepth[].x */
			{
				dydx[1] = exp((-dE_R/Tcritdepth[i].y)+Beta);	/* calc. dydx based on corresponding Tcritdepth[].y value */
				i = time_steps + 1;					/* exit from for() loop and return dydx to calling function */
			}
			else if (x < Tcritdepth[i].x && x > Tcritdepth[i-1].x) 	/* if x is less than current Tcritdepth but great than previous */
			{
				Tint[i].y = GenInt(Tcritdepth[i].x, x, Tcritdepth[i-1].x, Tcritdepth[i].y, Tcritdepth[i-1].y);
				dydx[1] = exp((-dE_R/Tint[i].y)+Beta);		/* calc. dydx based on interpolated T value */
				i = time_steps + 1;					/* exit from for() loop and return dydx to calling function */
			}
			else if (x > Tcritdepth[i].x && x < Tcritdepth[i+1].x) 	/* if x is greater than current Tcritdepth but less than previous */
			{
				Tint[i].y = GenInt(Tcritdepth[i+1].x, x, Tcritdepth[i].x, Tcritdepth[i+1].y, Tcritdepth[i].y);
				dydx[1] = exp((-dE_R/Tint[i].y)+Beta);		/* calc. dydx based on interpolated T value */
				i = time_steps + 1;					/* exit from for() loop and return dydx to calling function */
			}
			else continue;		/* if none of these conditions exist, continue with for() loop */
		} 
	}
}		/* End of derivs() */

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void rk4(double y[], double dydx[], int n, double x, double h, double yout[],
 *							 void (*derivs)(double, double [], double []))
 *
 *	Purpose: Works with rkdumb to solve integral
 *
 *	Arguments:	double y[] - 
 *				double dydx[] - 
 *				int n - 
 *				double x - 
 *				double h - 
 *				double yout[] - 
 *				void *derivs - see derivs() above
 *
 *	Return:  void
 *	
 *	Created:	4/8/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	int i -
 *						double xh -
 *						double hh -
 *						double h6 -
 *						double *dym -
 *						double *dyt -
 *						double *yt -
 *
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */

void rk4(double y[], double dydx[], int n, double x, double h, double yout[], void (*derivs)(double, double [], double [])) 
{
	int i;
	double xh,hh,h6,*dym,*dyt,*yt;

	dym=vector(1,n);
	dyt=vector(1,n);
	yt=vector(1,n);
	hh=h*0.5;
	h6=h/6.0;
	xh=x+hh;
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dydx[i];
	(*derivs)(xh,yt,dyt);
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dyt[i];
	(*derivs)(xh,yt,dym);
	for (i=1;i<=n;i++) 
	{
		yt[i]=y[i]+h*dym[i];
		dym[i] += dyt[i];
	}
	(*derivs)(x+h,yt,dyt);
	for (i=1;i<=n;i++)	yout[i]=y[i]+h6*(dydx[i]+dyt[i]+2.0*dym[i]);
	free_vector(yt,1,n);
	free_vector(dyt,1,n);
	free_vector(dym,1,n);

}		/* End of rk4()	*/

/* ----------------------------------------------------------------------------
 *
 *	Function Name:  void rkdumb(double vstart[], int nvar, double x1, double x2, int nstep,
 *								void (*derivs)(double, double [], double []))
 *
 *	Purpose: Stepper function that drives the rk4 integral solver
 *
 *	Arguments:	double vstart[] - 
 *				int nvar - 
 *				double x1 - 
 *				double x2 - 
 *				int nstep - 
 *				void *derivs - see derivs() above
 *
 *	Return:  void
 *	
 *	Created:	4/8/98	Numerical Recipes Copyright (C) 1986 - 1995
 *
 *	Local Variables:	int i, k -
 *						double x - 
 *						double h - 
 *						double *v - 
 *						double *vout - 
 *						double *dv - 
 *
 *	Global Variables modified:	*xx, **y
 *
 * ------------------------------------------------------------------------- */


void rkdumb(double vstart[], int nvar, double x1, double x2, int nstep, void (*derivs)(double, double [], double [])) 
{
	void rk4(double y[], double dydx[], int n, double x, double h, double yout[], void (*derivs)(double, double [], double []));
	int i,k;
	double x,h;
	double *v,*vout,*dv;

	v=vector(1,nvar);
	vout=vector(1,nvar);
	dv=vector(1,nvar);
	for (i=1;i<=nvar;i++) 
	{
		v[i]=vstart[i];
		y[i][1]=v[i];
	}
	xx[1]=x1;
	x=x1;
	h=(x2-x1)/nstep;
	for (k=1;k<=nstep;k++) 
	{
		(*derivs)(x,v,dv);
		rk4(v,dv,nvar,x,h,vout,derivs);
		if ((double)(x+h) == x) nrerror("Step size too small in routine rkdumb");
		x += h;
		xx[k+1]=x;
		for (i=1;i<=nvar;i++) 
		{
			v[i]=vout[i];
			y[i][k+1]=v[i];
		}
	}
	free_vector(dv,1,nvar);
	free_vector(vout,1,nvar);
	free_vector(v,1,nvar);

}		/* End of rkdumb() */


/*     **** BURN INJURY CRITERION FUNCTION, OMEGA ****
 * 
 *    The Henriques rate function injury model is used.  In this method
 *    it is required to calculate an injury parameter Omega to predict
 *    burn severity at a some critical cell depth in the skin, which may
 *    be assumed to be the basal epidermal layer.  Omega is in the form
 *    of an integral over time of a function of T(t), the temperature
 *    at the critical depth, and two empirically determined constants,
 *    delE and G.  The critical cell depth is a program input that
 *    should normally be specified as 72 micrometers(72 m, 0.000072 m).
 *
 *    Omega = 0.53 corresponds to the threshold of first-degree burn.
 *    Omega >= 1 corresponds to a minimum-severity, partial-skin-loss
 *    burn, in which the depth of injury has extended fully through
 *    the epidermis (incipient second-degree burn). 
 *	 
 *	 Taken from B. Bowman Fortan Burn Model Program
 ---------------------------------------------------------------------*
 *
 *	Function Name:  int BurnInjury(void)
 *
 *	Purpose: uses the Tcritdepth values to calculate the Burn Integral
 *
 *	Arguments:	none
 *
 *	Return:  Integer to main if completes successfully
 *	
 *	Created:	4/10/98	SJR
 *	Modified:			Modification: 
 *	Modified:			Modification:	
 *
 *	Local Variables:	int nstep, expn, npower -
 *						int nbias, nlog - 
 *						double bias, fbias - 
 *						double G, Gc - 
 *						double *vstart, A - 
 *						double integral, thelog, flog, Omega - 
 *						char junk[85] - 
 *						FILE *fpomega - 
 *
 *	Global Variables modified:	
 *
 * ------------------------------------------------------------------------- */

double BurnInjury(void) 
{
	int nstep, nbias, nlog, npower;
	double bias, fbias, G;
	double *vstart;
	double integral, thelog, flog;

	nstep = time_steps;		/* set the Runge-Kutta NSTEP equal to the time steps of the system */

	vstart=vector(1,NVAR);		/* create the memory vector */
	Tbar=(Tcritdepth[1].y+Tcritdepth[time_steps].y)/2;		/* Tbar is the Average Tcritdepth over the entire time interval */
	dE_R = dE / R;		/* deltaE / R is a constant */
	Beta = dE_R / Tbar;		/* deltaE / R * Tbar is then calculated and used in the Burn Integral calculation */

/* Start building Omega = G * Integral where G = Gc * 10^-(bias-nbias) * 10^(thelog-nlog) * 10^((expn-nbias)+nlog)   */

	bias = Beta * log10(exp(1));		/* calculate the double type value of Beta*log10e value */
	nbias = Beta * log10(exp(1));		/* set nbias to Integer value of Beta*log10e */
	fbias = bias - nbias;				/* calc the double type decimal difference between the biases */
    G = Gc * pow(10,-fbias);			/* start calculating G by incrementally multiplying powers of 10 */
    npower = expn - nbias;				/* calc. a power of 10 for G */

/* Note: The arrays xx and y must have indices up to NSTEP+1 */
	xx=vector(1,nstep+1);				/* create memory vector for xx pointer */
	y=matrix(1,NVAR,1,nstep+1);			/* create memory matrix for y pointer */
	vstart[1]=0.00;						/* set initial value variable to zero: [1] denotes the first order system */
	
	Tcritdepth[0].x = 0.00;				/* set critical depth time/temp @ t= zero */
	Tcritdepth[0].y = T[printnode].y;	/* set Temp @ time zero @ critical depth = initial Temp distribution @ node */
	
	rkdumb(vstart, NVAR, Tcritdepth[0].x, Tcritdepth[time_steps].x, nstep, derivs);		/* call NR function to integrate */
		
	integral = y[1][nstep];				/* get value of Runge-Kutta integration at end time point */
	thelog = log10(integral);			/* take log base-10 or integral value: for further reduction of Omega calc. */
	nlog = thelog;						/* set nlog to integer value of thelog */
	flog = thelog - nlog;				/* find decimal remainder of thelog */
	G = G * pow(10,flog);				/* perform second multiplication for G */
	npower = npower + nlog;				/* increase npower by nlog to reduce eponential power further */
	
	Omega = G * pow(10,npower);			/* perform last multiplication for G  */
	
	if (Omega < .000001) Omega = 0.000000;

	free_matrix(y,1,NVAR,1,nstep+1);	/* release matrix memory */
	free_vector(xx,1,nstep+1);			/* release vector memory */
	free_vector(vstart,1,NVAR);			/* release vector memory */
	
	#undef NRANSI									/* undefine NRANSI */

	return Omega;

}		/* End of BurnInjury() */





/* End of Code */
