/* File: BURN.H */

/* Include the necessary libraries and definitions for standard and tridiagonal functions */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#define NRANSI
#include "nr.h"
#include "nrutil.h"
/*		End of #include Declaration */

/* Define specific constants */
#define MAXARRAY 30001							/* sets the maximum number of nodes and time steps */
#define NP MAXARRAY								/* must be equal to the maximum possible array size */
#define NR_END 1								/* NR specific constant */
#define FREE_ARG char*							/* NR specific #define */
#define NVAR 1									/* ODE order of the system, always 1 for Burn Model */
#define one 1.000000000000
#define negone -1.000000000000
#define two 2.000000000000
#define four 4.000000000000

/*		End of #define Declaration */

/* Function Declarations */
double BurnInjury(void);							/* calculate the value of Omega */
void rkdumb(double vstart[], int nvar, double x1, double x2, int nstep, void (*derivs)(double, double [], double []));/*rk4 stepper func */
void rk4(double y[], double dydx[], int n, double x, double h, double yout[], void (*derivs)(double, double [], double []));/*integral func*/
void derivs(double x,double y[],double dydx[]);	/* calculate the first derivative of the T function at each time step */
void tridag(double a[], double b[], double c[], double r[], double u[], unsigned long n);	/* Tridiagonal solver function */
void InterRHS(void);							/* build the interior node vlaues in the RHS matrix */
void Flux(char bc, int step);					/* calculate the flux values for node 0 and nodes-1, for either LBC or RBC*/
void Convection(char bc, int step);				/* calculate the convection values for node 0 and nodes-1, for either LBC or RBC*/
int InterNodes(void);							/* calculate the interior node values for the A-matrix */
int Fourier(void);								/* calculate the Fo numbers for all interior nodes */
void Interp(char *pp, int sets);				/* calculate the interpolated values for all necessary inputs */
double JetConvection(int incr, double temp, double amb);	/* calculate hbar used for impinging jet convection model */
double GenInt(double a, double b, double c, double d, double e);  /* Generic Interpolation function */
int GetData(void);								/* function used to read the input data */
void Title(void);
/*		End of Function Declaration	*/


/* Declare Global Variables */
char datadir[60];								/* command line input for pointing the program to the data directory */

struct datapair {
	double x;									/* all Burn Model variables use type 'double,' eventhough NR only allows 'double' */
	double y;
};												/* generic structure for reading and using pairs of data */

struct datapair conduct[MAXARRAY];				/* depth, conduction array, one set per node */
struct datapair heatcap[MAXARRAY];				/* depth, Cp */
struct datapair density[MAXARRAY];				/* depth, p (rho) */
struct datapair lbc1[MAXARRAY];	   /* time, temp flux|convection coefficient for l.b.c. array, one set for each node (1 and nodes) */
struct datapair lbc2[MAXARRAY];					/* time, ambient temp for left boundary condition */
struct datapair rbc1[MAXARRAY];					/* time, temp|flux|convection coefficient for right boundary condition */
struct datapair rbc2[MAXARRAY];					/* time, ambient temp for right boundary condition */
struct datapair T[MAXARRAY];					/* time, node temperature, in RHS matrix */
struct datapair Tcritdepth[MAXARRAY];			/* time and T values at critical depth, for Burn Integral calculations */
struct datapair gasV[MAXARRAY];					/* Velocity Time inputs for Jet Convection model */
struct datapair	jetdur[MAXARRAY];				/* boundary condition value calculated for impinging jet*/

double **rptr;
double *aptr;

double diag[MAXARRAY];							/* A-matrix diagonal values, nodes x nodes matrix */
double upper[MAXARRAY];							/* A-matrix upper diagonal values */
double lower[MAXARRAY];							/* A-matrix lower diagonal values */
double rhs[MAXARRAY];							/* C-matrix values, 1 x nodes matrix */

int jet;										/* global flag indicating Impinging Jet convection is being used */
double dur;										/* used only for finding Hbar duration where Omega = 1 */
double Reknown;										/* Reynolds number used for Impinging Jet Convection model */
double Tout;									/* Outlet Temperature of impingin jet */
double onaxisbump;									/* Used to signal use of Onaxis Bump term in calculation */
double onaxis;									/* On-axis Spacing for impinging jet */
double offaxis;									/* Off-axis spacing for imping jet */
double ventD;									/* Impinging Jet Vent Diamete */
double hbar;									/* resulting value from Jet Convection model */
double altgas;									/* allow user to provide alternate gas characteristics */
double altgasK;									/* allow user to provide alternate gas characteristics */
double altgasPr;									/* allow user to provide alternate gas characteristics */
double altgasVisc;									/* allow user to provide alternate gas characteristics */
				
double fo[MAXARRAY];							/* fourier number array for nodes*/
double bifo[MAXARRAY];							/* BiFo number array for nodes */

double printtime;								/* output parameter for Temp vs. Depth at a specific time */
int printnode;									/* output parameter for Temp vs. time at a specific node/depth  */
int VERBOSE;									/* 1 for output file; 0 for screen */
int BATCH;
int printA, printI;								/* conditionally used to either print or not print A-matrix and Initial Node Temp */
int multio;										/* 1 to find the node where Omega = 1; 0 for single run */
int trial;										/* used to mark the output file with the same trial number as the data.dat file */
char date[20], inits[10];

double dE_R, Tbar, Beta;						/* Burn Integral constants */
double dE, R, Gc;								/* Burn Integral constants */
int expn;										/* Burn Integral constant */

char lbc_type[20], rbc_type[20];				/* contains the left and right boundary condition type for switching in main()*/
int nodes, time_steps;							/* number of nodes in system, number of time steps for solver */
double dx, dt, t;								/* mid-segment-mid-segment distance, time increment, total time */ 
double Omega;									/* Burn integral output value */

double **y, *xx;									/* NR requires these for rk4 output */
/*		End of Global Variable declaration */
